@@ -2,7 +2,7 @@
 /*
  * Copyright (c) 2012-2016 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, LLC. All rights reserved
- * Copyright (c) 2015-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -190,6 +190,7 @@ static inline pmix_status_t pmix_hotel_checkin(pmix_hotel_t *hotel,
 
     /* Do we have any rooms available? */
     if (PMIX_UNLIKELY(hotel->last_unoccupied_room < 0)) {
+        *room_num = -1;
         return PMIX_ERR_OUT_OF_RESOURCE;
     }
 
@@ -247,6 +248,10 @@ static inline void pmix_hotel_checkout(pmix_hotel_t *hotel, int room_num)
 
     /* Bozo check */
     assert(room_num < hotel->num_rooms);
+    if (0 > room_num) {
+        /* occupant wasn't checked in */
+        return;
+    }
 
     /* If there's an occupant in the room, check them out */
     room = &(hotel->rooms[room_num]);
@@ -285,6 +290,11 @@ static inline void pmix_hotel_checkout_and_return_occupant(pmix_hotel_t *hotel,
 
     /* Bozo check */
     assert(room_num < hotel->num_rooms);
+    if (0 > room_num) {
+        /* occupant wasn't checked in */
+        *occupant = NULL;
+        return;
+    }
 
     /* If there's an occupant in the room, check them out */
     room = &(hotel->rooms[room_num]);
@@ -339,6 +349,10 @@ static inline void pmix_hotel_knock(pmix_hotel_t *hotel, int room_num, void **oc
     assert(room_num < hotel->num_rooms);
 
     *occupant = NULL;
+    if (0 > room_num) {
+        /* occupant wasn't checked in */
+        return;
+    }
 
     /* If there's an occupant in the room, have them come to the door */
     room = &(hotel->rooms[room_num]);
