@@ -1,9 +1,9 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014-2017 Research Organization for Information Science
- * Copyright (c) 2014-2018 Intel, Inc.  All rights reserved.
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2014-2015 Artem Y. Polyakov <artpol84@gmail.com>.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
@@ -96,7 +96,7 @@ PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_cleanup_dir_t,
                                 pmix_list_item_t,
                                 cdcon, cddes);
 
-static void nscon(pmix_nspace_t *p)
+static void nscon(pmix_namespace_t *p)
 {
     p->nspace = NULL;
     p->nprocs = 0;
@@ -113,7 +113,7 @@ static void nscon(pmix_nspace_t *p)
     PMIX_CONSTRUCT(&p->epilog.ignores, pmix_list_t);
     PMIX_CONSTRUCT(&p->setup_data, pmix_list_t);
 }
-static void nsdes(pmix_nspace_t *p)
+static void nsdes(pmix_namespace_t *p)
 {
     if (NULL != p->nspace) {
         free(p->nspace);
@@ -130,7 +130,7 @@ static void nsdes(pmix_nspace_t *p)
     PMIX_LIST_DESTRUCT(&p->epilog.ignores);
     PMIX_LIST_DESTRUCT(&p->setup_data);
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_nspace_t,
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_namespace_t,
                                 pmix_list_item_t,
                                 nscon, nsdes);
 
@@ -238,6 +238,9 @@ static void iofreqdes(pmix_iof_req_t *p)
     if (NULL != p->peer) {
         PMIX_RELEASE(p->peer);
     }
+    if (NULL != p->pname.nspace) {
+        free(p->pname.nspace);
+    }
 }
 PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_iof_req_t,
                                 pmix_list_item_t,
@@ -344,6 +347,8 @@ static void qdes(pmix_query_caddy_t *p)
 {
     PMIX_DESTRUCT_LOCK(&p->lock);
     PMIX_BYTE_OBJECT_DESTRUCT(&p->bo);
+    PMIX_PROC_FREE(p->targets, p->ntargets);
+    PMIX_INFO_FREE(p->info, p->ninfo);
 }
 PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_query_caddy_t,
                                 pmix_object_t,
@@ -355,32 +360,38 @@ void pmix_execute_epilog(pmix_epilog_t *epi)
     pmix_cleanup_dir_t *cd, *cdnext;
     struct stat statbuf;
     int rc;
+    char **tmp;
+    size_t n;
 
     /* start with any specified files */
     PMIX_LIST_FOREACH_SAFE(cf, cfnext, &epi->cleanup_files, pmix_cleanup_file_t) {
         /* check the effective uid/gid of the file and ensure it
          * matches that of the peer - we do this to provide at least
          * some minimum level of protection */
-        rc = stat(cf->path, &statbuf);
-        if (0 != rc) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "File %s failed to stat: %d", cf->path, rc);
-            continue;
-        }
-        if (statbuf.st_uid != epi->uid ||
-            statbuf.st_gid != epi->gid) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "File %s uid/gid doesn't match: uid %lu(%lu) gid %lu(%lu)",
-                                cf->path,
-                                (unsigned long)statbuf.st_uid, (unsigned long)epi->uid,
-                                (unsigned long)statbuf.st_gid, (unsigned long)epi->gid);
-            continue;
-        }
-        rc = unlink(cf->path);
-        if (0 != rc) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "File %s failed to unlink: %d", cf->path, rc);
+        tmp = pmix_argv_split(cf->path, ',');
+        for (n=0; NULL != tmp[n]; n++) {
+            rc = stat(tmp[n], &statbuf);
+            if (0 != rc) {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "File %s failed to stat: %d", tmp[n], rc);
+                continue;
+            }
+            if (statbuf.st_uid != epi->uid ||
+                statbuf.st_gid != epi->gid) {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "File %s uid/gid doesn't match: uid %lu(%lu) gid %lu(%lu)",
+                                    cf->path,
+                                    (unsigned long)statbuf.st_uid, (unsigned long)epi->uid,
+                                    (unsigned long)statbuf.st_gid, (unsigned long)epi->gid);
+                continue;
+            }
+            rc = unlink(tmp[n]);
+            if (0 != rc) {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "File %s failed to unlink: %d", tmp[n], rc);
+            }
         }
+        pmix_argv_free(tmp);
         pmix_list_remove_item(&epi->cleanup_files, &cf->super);
         PMIX_RELEASE(cf);
     }
@@ -390,27 +401,31 @@ void pmix_execute_epilog(pmix_epilog_t *epi)
         /* check the effective uid/gid of the file and ensure it
          * matches that of the peer - we do this to provide at least
          * some minimum level of protection */
-        rc = stat(cd->path, &statbuf);
-        if (0 != rc) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "Directory %s failed to stat: %d", cd->path, rc);
-            continue;
-        }
-        if (statbuf.st_uid != epi->uid ||
-            statbuf.st_gid != epi->gid) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "Directory %s uid/gid doesn't match: uid %lu(%lu) gid %lu(%lu)",
-                                cd->path,
-                                (unsigned long)statbuf.st_uid, (unsigned long)epi->uid,
-                                (unsigned long)statbuf.st_gid, (unsigned long)epi->gid);
-            continue;
-        }
-        if ((statbuf.st_mode & S_IRWXU) == S_IRWXU) {
-            dirpath_destroy(cd->path, cd, epi);
-        } else {
-            pmix_output_verbose(10, pmix_globals.debug_output,
-                                "Directory %s lacks permissions", cd->path);
+        tmp = pmix_argv_split(cd->path, ',');
+        for (n=0; NULL != tmp[n]; n++) {
+            rc = stat(tmp[n], &statbuf);
+            if (0 != rc) {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "Directory %s failed to stat: %d", tmp[n], rc);
+                continue;
+            }
+            if (statbuf.st_uid != epi->uid ||
+                statbuf.st_gid != epi->gid) {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "Directory %s uid/gid doesn't match: uid %lu(%lu) gid %lu(%lu)",
+                                    cd->path,
+                                    (unsigned long)statbuf.st_uid, (unsigned long)epi->uid,
+                                    (unsigned long)statbuf.st_gid, (unsigned long)epi->gid);
+                continue;
+            }
+            if ((statbuf.st_mode & S_IRWXU) == S_IRWXU) {
+                dirpath_destroy(tmp[n], cd, epi);
+            } else {
+                pmix_output_verbose(10, pmix_globals.debug_output,
+                                    "Directory %s lacks permissions", tmp[n]);
+            }
         }
+        pmix_argv_free(tmp);
         pmix_list_remove_item(&epi->cleanup_dirs, &cd->super);
         PMIX_RELEASE(cd);
     }
