@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 2018      Intel, Inc. All rights reserved.
+ * Copyright (c) 2018-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -47,17 +49,17 @@
 
 static pmix_status_t tcp_init(void);
 static void tcp_finalize(void);
-static pmix_status_t allocate(pmix_nspace_t *nptr,
+static pmix_status_t allocate(pmix_namespace_t *nptr,
                               pmix_info_t *info,
                               pmix_list_t *ilist);
-static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
+static pmix_status_t setup_local_network(pmix_namespace_t *nptr,
                                          pmix_info_t info[],
                                          size_t ninfo);
-static pmix_status_t setup_fork(pmix_nspace_t *nptr,
+static pmix_status_t setup_fork(pmix_namespace_t *nptr,
                                 const pmix_proc_t *peer, char ***env);
 static void child_finalized(pmix_proc_t *peer);
-static void local_app_finalized(pmix_nspace_t *nptr);
-static void deregister_nspace(pmix_nspace_t *nptr);
+static void local_app_finalized(pmix_namespace_t *nptr);
+static void deregister_nspace(pmix_namespace_t *nptr);
 static pmix_status_t collect_inventory(pmix_info_t directives[], size_t ndirs,
                                        pmix_inventory_cbfunc_t cbfunc, void *cbdata);
 static pmix_status_t deliver_inventory(pmix_info_t info[], size_t ninfo,
@@ -102,7 +104,7 @@ typedef struct {
 } tcp_port_tracker_t;
 
 static pmix_list_t allocations, available;
-static pmix_status_t process_request(pmix_nspace_t *nptr,
+static pmix_status_t process_request(pmix_namespace_t *nptr,
                                      char *idkey, int ports_per_node,
                                      tcp_port_tracker_t *trk,
                                      pmix_list_t *ilist);
@@ -295,7 +297,7 @@ static inline void generate_key(uint64_t* unique_key) {
  * NOTE: this implementation is offered as an example that can
  * undoubtedly be vastly improved/optimized */
 
-static pmix_status_t allocate(pmix_nspace_t *nptr,
+static pmix_status_t allocate(pmix_namespace_t *nptr,
                               pmix_info_t *info,
                               pmix_list_t *ilist)
 {
@@ -329,16 +331,20 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
     /* check directives to see if a crypto key and/or
      * network resource allocations requested */
     PMIX_CONSTRUCT(&mylist, pmix_list_t);
-    if (0 == strncmp(info->key, PMIX_SETUP_APP_ENVARS, PMIX_MAX_KEYLEN) ||
-        0 == strncmp(info->key, PMIX_SETUP_APP_ALL, PMIX_MAX_KEYLEN)) {
+    if (PMIX_CHECK_KEY(info, PMIX_SETUP_APP_ENVARS) ||
+        PMIX_CHECK_KEY(info, PMIX_SETUP_APP_ALL)) {
         if (NULL != mca_pnet_tcp_component.include) {
+        pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                            "pnet: tcp harvesting envars %s excluding %s",
+                            (NULL == mca_pnet_tcp_component.incparms) ? "NONE" : mca_pnet_tcp_component.incparms,
+                            (NULL == mca_pnet_tcp_component.excparms) ? "NONE" : mca_pnet_tcp_component.excparms);
             rc = pmix_pnet_base_harvest_envars(mca_pnet_tcp_component.include,
                                                mca_pnet_tcp_component.exclude,
                                                ilist);
             return rc;
         }
         return PMIX_SUCCESS;
-    } else if (0 != strncmp(info->key, PMIX_ALLOC_NETWORK, PMIX_MAX_KEYLEN)) {
+    } else if (!PMIX_CHECK_KEY(info, PMIX_ALLOC_NETWORK)) {
         /* not a network allocation request */
         return PMIX_SUCCESS;
     }
@@ -443,11 +449,13 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
             }
             /* nope - they asked for something that we cannot do */
             if (NULL == avail) {
+                PMIX_LIST_DESTRUCT(&mylist);
                 return PMIX_ERR_NOT_AVAILABLE;
             }
             /* setup to track the assignment */
             trk = PMIX_NEW(tcp_port_tracker_t);
             if (NULL == trk) {
+                PMIX_LIST_DESTRUCT(&mylist);
                 return PMIX_ERR_NOMEM;
             }
             trk->nspace = strdup(nptr->nspace);
@@ -459,6 +467,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                 /* return the allocated ports */
                 pmix_list_remove_item(&allocations, &trk->super);
                 PMIX_RELEASE(trk);
+                PMIX_LIST_DESTRUCT(&mylist);
                 return rc;
             }
             allocated = true;
@@ -481,11 +490,13 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
             }
             /* nope - they asked for something that we cannot do */
             if (NULL == avail) {
+                PMIX_LIST_DESTRUCT(&mylist);
                 return PMIX_ERR_NOT_AVAILABLE;
             }
             /* setup to track the assignment */
             trk = PMIX_NEW(tcp_port_tracker_t);
             if (NULL == trk) {
+                PMIX_LIST_DESTRUCT(&mylist);
                 return PMIX_ERR_NOMEM;
             }
             trk->nspace = strdup(nptr->nspace);
@@ -497,6 +508,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                 /* return the allocated ports */
                 pmix_list_remove_item(&allocations, &trk->super);
                 PMIX_RELEASE(trk);
+                PMIX_LIST_DESTRUCT(&mylist);
                 return rc;
             }
             allocated = true;
@@ -505,6 +517,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
             pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
                                 "pnet:tcp:allocate unsupported type %s for nspace %s",
                                 type, nptr->nspace);
+            PMIX_LIST_DESTRUCT(&mylist);
             return PMIX_ERR_TAKE_NEXT_OPTION;
         }
 
@@ -519,6 +532,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                 /* setup to track the assignment */
                 trk = PMIX_NEW(tcp_port_tracker_t);
                 if (NULL == trk) {
+                    PMIX_LIST_DESTRUCT(&mylist);
                     return PMIX_ERR_NOMEM;
                 }
                 trk->nspace = strdup(nptr->nspace);
@@ -530,6 +544,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                     /* return the allocated ports */
                     pmix_list_remove_item(&allocations, &trk->super);
                     PMIX_RELEASE(trk);
+                    PMIX_LIST_DESTRUCT(&mylist);
                     return rc;
                 }
                 allocated = true;
@@ -583,6 +598,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                     trk = PMIX_NEW(tcp_port_tracker_t);
                     if (NULL == trk) {
                         pmix_argv_free(reqs);
+                        PMIX_LIST_DESTRUCT(&mylist);
                         return PMIX_ERR_NOMEM;
                     }
                     trk->nspace = strdup(nptr->nspace);
@@ -594,6 +610,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                         /* return the allocated ports */
                         pmix_list_remove_item(&allocations, &trk->super);
                         PMIX_RELEASE(trk);
+                        PMIX_LIST_DESTRUCT(&mylist);
                         return rc;
                     }
                     allocated = true;
@@ -604,6 +621,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                                     ports_per_node, nptr->nspace);
                 if (0 == ports_per_node) {
                     /* nothing to allocate */
+                    PMIX_LIST_DESTRUCT(&mylist);
                     return PMIX_ERR_TAKE_NEXT_OPTION;
                 }
                 avail = (tcp_available_ports_t*)pmix_list_get_first(&available);
@@ -611,6 +629,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
                     /* setup to track the assignment */
                     trk = PMIX_NEW(tcp_port_tracker_t);
                     if (NULL == trk) {
+                        PMIX_LIST_DESTRUCT(&mylist);
                         return PMIX_ERR_NOMEM;
                     }
                     trk->nspace = strdup(nptr->nspace);
@@ -630,26 +649,32 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
         }
         if (!allocated) {
             /* nope - we cannot help */
+            PMIX_LIST_DESTRUCT(&mylist);
             return PMIX_ERR_TAKE_NEXT_OPTION;
         }
     }
 
     if (seckey) {
+        pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                            "pnet:tcp: generate seckey");
         generate_key(unique_key);
         kv = PMIX_NEW(pmix_kval_t);
         if (NULL == kv) {
+            PMIX_LIST_DESTRUCT(&mylist);
             return PMIX_ERR_NOMEM;
         }
         kv->key = strdup(PMIX_ALLOC_NETWORK_SEC_KEY);
         kv->value = (pmix_value_t*)malloc(sizeof(pmix_value_t));
         if (NULL == kv->value) {
             PMIX_RELEASE(kv);
+            PMIX_LIST_DESTRUCT(&mylist);
             return PMIX_ERR_NOMEM;
         }
         kv->value->type = PMIX_BYTE_OBJECT;
         kv->value->data.bo.bytes = (char*)malloc(2 * sizeof(uint64_t));
         if (NULL == kv->value->data.bo.bytes) {
             PMIX_RELEASE(kv);
+            PMIX_LIST_DESTRUCT(&mylist);
             return PMIX_ERR_NOMEM;
         }
         memcpy(kv->value->data.bo.bytes, unique_key, 2 * sizeof(uint64_t));
@@ -696,7 +721,7 @@ static pmix_status_t allocate(pmix_nspace_t *nptr,
 /* upon receipt of the launch message, each daemon adds the
  * static address assignments to the job-level info cache
  * for that job */
-static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
+static pmix_status_t setup_local_network(pmix_namespace_t *nptr,
                                          pmix_info_t info[],
                                          size_t ninfo)
 {
@@ -724,7 +749,12 @@ static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
                 PMIX_BFROPS_UNPACK(rc, pmix_globals.mypeer,
                                    &bkt, &nkvals, &cnt, PMIX_SIZE);
                 /* setup the info array */
-                PMIX_INFO_CREATE(jinfo, nkvals);
+                PMIX_INFO_CONSTRUCT(&stinfo);
+                pmix_strncpy(stinfo.key, idkey, PMIX_MAX_KEYLEN);
+                stinfo.value.type = PMIX_DATA_ARRAY;
+                PMIX_DATA_ARRAY_CREATE(stinfo.value.data.darray, nkvals, PMIX_INFO);
+                jinfo = (pmix_info_t*)stinfo.value.data.darray->array;
+
                 /* cycle thru the blob and extract the kvals */
                 kv = PMIX_NEW(pmix_kval_t);
                 cnt = 1;
@@ -736,7 +766,7 @@ static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
                                         "recvd KEY %s %s", kv->key,
                                         (PMIX_STRING == kv->value->type) ? kv->value->data.string : "NON-STRING");
                     /* xfer the value to the info */
-                    (void)strncpy(jinfo[m].key, kv->key, PMIX_MAX_KEYLEN);
+                    pmix_strncpy(jinfo[m].key, kv->key, PMIX_MAX_KEYLEN);
                     PMIX_BFROPS_VALUE_XFER(rc, pmix_globals.mypeer,
                                            &jinfo[m].value, kv->value);
                     /* if this is the ID key, save it */
@@ -762,12 +792,6 @@ static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
                     PMIX_INFO_FREE(jinfo, nkvals);
                     return PMIX_ERR_BAD_PARAM;
                 }
-                /* the data gets stored as a pmix_data_array_t on the provided key */
-                PMIX_INFO_CONSTRUCT(&stinfo);
-                (void)strncpy(stinfo.key, idkey, PMIX_MAX_KEYLEN);
-                stinfo.value.type = PMIX_DATA_ARRAY;
-                PMIX_DATA_ARRAY_CREATE(stinfo.value.data.darray, nkvals, PMIX_INFO);
-                stinfo.value.data.darray->array = jinfo;
 
                 /* cache the info on the job */
                 PMIX_GDS_CACHE_JOB_INFO(rc, pmix_globals.mypeer, nptr,
@@ -782,9 +806,11 @@ static pmix_status_t setup_local_network(pmix_nspace_t *nptr,
     return PMIX_SUCCESS;
 }
 
-static pmix_status_t setup_fork(pmix_nspace_t *nptr,
+static pmix_status_t setup_fork(pmix_namespace_t *nptr,
                                 const pmix_proc_t *peer, char ***env)
 {
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet:tcp:setup_fork");
     return PMIX_SUCCESS;
 }
 
@@ -801,7 +827,7 @@ static void child_finalized(pmix_proc_t *peer)
  * provides an opportunity for the local network to cleanup
  * any resources consumed locally by the clients of that job.
  * We don't have anything we need to do */
-static void local_app_finalized(pmix_nspace_t *nptr)
+static void local_app_finalized(pmix_namespace_t *nptr)
 {
     pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
                         "pnet:tcp app finalized");
@@ -811,7 +837,7 @@ static void local_app_finalized(pmix_nspace_t *nptr)
  * PMix function, which in turn calls my TCP component to release the
  * assignments for that job. The addresses are marked as "available"
  * for reuse on the next job. */
-static void deregister_nspace(pmix_nspace_t *nptr)
+static void deregister_nspace(pmix_namespace_t *nptr)
 {
     tcp_port_tracker_t *trk;
 
@@ -852,6 +878,9 @@ static pmix_status_t collect_inventory(pmix_info_t directives[], size_t ndirs,
     pmix_byte_object_t pbo;
     pmix_kval_t *kv;
 
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet:tcp:collect_inventory");
+
     /* setup the bucket - we will pass the results as a blob */
     PMIX_CONSTRUCT(&bucket, pmix_buffer_t);
     /* add our hostname */
@@ -949,7 +978,7 @@ static pmix_status_t collect_inventory(pmix_info_t directives[], size_t ndirs,
     return PMIX_SUCCESS;
 }
 
-static pmix_status_t process_request(pmix_nspace_t *nptr,
+static pmix_status_t process_request(pmix_namespace_t *nptr,
                                      char *idkey, int ports_per_node,
                                      tcp_port_tracker_t *trk,
                                      pmix_list_t *ilist)
