@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2016-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2018      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -48,10 +49,13 @@ typedef struct {
     char *session_filename;
     char *nspace_filename;
     char *system_filename;
+    char *rendezvous_filename;
     int wait_to_connect;
     int max_retries;
     char *report_uri;
     bool remote_connections;
+    int handshake_wait_time;
+    int handshake_max_retries;
 } pmix_ptl_tcp_component_t;
 
 extern pmix_ptl_tcp_component_t mca_ptl_tcp_component;
