@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2014      Artem Y. Polyakov <artpol84@gmail.com>.
@@ -63,6 +63,7 @@
 #include "src/runtime/pmix_rte.h"
 #include "src/mca/bfrops/base/base.h"
 #include "src/mca/gds/base/base.h"
+#include "src/mca/pnet/base/base.h"
 #include "src/mca/ptl/base/base.h"
 #include "src/mca/psec/psec.h"
 #include "src/include/pmix_globals.h"
@@ -162,30 +163,9 @@ static void pmix_tool_notify_recv(struct pmix_peer_t *peer,
             PMIX_RELEASE(chain);
             goto error;
         }
-        /* check for directives */
-        for (cnt=0; cnt < (int)ninfo; cnt++) {
-            if (0 == strncmp(chain->info[cnt].key, PMIX_EVENT_NON_DEFAULT, PMIX_MAX_KEYLEN)) {
-                chain->nondefault = PMIX_INFO_TRUE(&chain->info[cnt]);
-            } else if (0 == strncmp(chain->info[cnt].key, PMIX_EVENT_AFFECTED_PROC, PMIX_MAX_KEYLEN)) {
-                PMIX_PROC_CREATE(chain->affected, 1);
-                if (NULL == chain->affected) {
-                    PMIX_RELEASE(chain);
-                    goto error;
-                }
-                chain->naffected = 1;
-                memcpy(chain->affected, chain->info[cnt].value.data.proc, sizeof(pmix_proc_t));
-            } else if (0 == strncmp(chain->info[cnt].key, PMIX_EVENT_AFFECTED_PROCS, PMIX_MAX_KEYLEN)) {
-                chain->naffected = chain->info[cnt].value.data.darray->size;
-                PMIX_PROC_CREATE(chain->affected, chain->naffected);
-                if (NULL == chain->affected) {
-                    chain->naffected = 0;
-                    PMIX_RELEASE(chain);
-                    goto error;
-                }
-                memcpy(chain->affected, chain->info[cnt].value.data.darray->array, chain->naffected * sizeof(pmix_proc_t));
-            }
-        }
     }
+    /* prep the chain for processing */
+    pmix_prep_event_chain(chain, chain->info, ninfo, false);
 
     pmix_output_verbose(2, pmix_client_globals.event_output,
                         "[%s:%d] pmix:tool_notify_recv - processing event %s from source %s:%d, calling errhandler",
@@ -216,7 +196,7 @@ static void tool_iof_handler(struct pmix_peer_t *pr,
     pmix_status_t rc;
 
     pmix_output_verbose(2, pmix_client_globals.iof_output,
-                        "recvd IOF");
+                        "recvd IOF with %d bytes", (int)buf->bytes_used);
 
     /* if the buffer is empty, they are simply closing the channel */
     if (0 == buf->bytes_used) {
@@ -311,7 +291,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
          * rank should be known. So return them here if
          * requested */
         if (NULL != proc) {
-            (void)strncpy(proc->nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
+            pmix_strncpy(proc->nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
             proc->rank = pmix_globals.myid.rank;
         }
         ++pmix_globals.init_cntr;
@@ -348,10 +328,29 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
                 /* they want us to forward our stdin to someone */
                 fwd_stdin = true;
             } else if (0 == strncmp(info[n].key, PMIX_LAUNCHER, PMIX_MAX_KEYLEN)) {
-                ptype = PMIX_PROC_LAUNCHER;
+                ptype |= PMIX_PROC_LAUNCHER;
+            } else if (0 == strncmp(info[n].key, PMIX_SERVER_TMPDIR, PMIX_MAX_KEYLEN)) {
+                pmix_server_globals.tmpdir = strdup(info[n].value.data.string);
+            } else if (0 == strncmp(info[n].key, PMIX_SYSTEM_TMPDIR, PMIX_MAX_KEYLEN)) {
+                pmix_server_globals.system_tmpdir = strdup(info[n].value.data.string);
             }
         }
     }
+    if (NULL == pmix_server_globals.tmpdir) {
+        if (NULL == (evar = getenv("PMIX_SERVER_TMPDIR"))) {
+            pmix_server_globals.tmpdir = strdup(pmix_tmp_directory());
+        } else {
+            pmix_server_globals.tmpdir = strdup(evar);
+        }
+    }
+    if (NULL == pmix_server_globals.system_tmpdir) {
+        if (NULL == (evar = getenv("PMIX_SYSTEM_TMPDIR"))) {
+            pmix_server_globals.system_tmpdir = strdup(pmix_tmp_directory());
+        } else {
+            pmix_server_globals.system_tmpdir = strdup(evar);
+        }
+    }
+
     if ((nspace_given && !rank_given) ||
         (!nspace_given && rank_given)) {
         /* can't have one and not the other */
@@ -408,7 +407,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
 
     /* if we are a launcher, then we also need to act as a server,
      * so setup the server-related structures here */
-    if (PMIX_PROC_LAUNCHER == ptype) {
+    if (PMIX_PROC_LAUNCHER_ACT & ptype) {
         if (PMIX_SUCCESS != (rc = pmix_server_initialize())) {
             PMIX_ERROR_LOG(rc);
             if (NULL != nspace) {
@@ -422,14 +421,6 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
         }
         /* setup the function pointers */
         memset(&pmix_host_server, 0, sizeof(pmix_server_module_t));
-        /* setup our tmpdir */
-        if (NULL == pmix_server_globals.tmpdir) {
-            if (NULL == (evar = getenv("PMIX_SERVER_TMPDIR"))) {
-                pmix_server_globals.tmpdir = strdup(pmix_tmp_directory());
-            } else {
-                pmix_server_globals.tmpdir = strdup(evar);
-            }
-        }
     }
 
     /* setup the runtime - this init's the globals,
@@ -448,7 +439,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
     }
     /* if we were given a name, then set it now */
     if (nspace_given || nspace_in_enviro) {
-        (void)strncpy(pmix_globals.myid.nspace, nspace, PMIX_MAX_NSLEN);
+        pmix_strncpy(pmix_globals.myid.nspace, nspace, PMIX_MAX_NSLEN);
         free(nspace);
         pmix_globals.myid.rank = rank;
     }
@@ -473,7 +464,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
         PMIX_RELEASE_THREAD(&pmix_global_lock);
         return PMIX_ERR_NOMEM;
     }
-    pmix_client_globals.myserver->nptr = PMIX_NEW(pmix_nspace_t);
+    pmix_client_globals.myserver->nptr = PMIX_NEW(pmix_namespace_t);
     if (NULL == pmix_client_globals.myserver->nptr) {
         PMIX_RELEASE(pmix_client_globals.myserver);
         if (gdsfound) {
@@ -603,7 +594,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
     }
     if (!nspace_given) {
         /* Success, so copy the nspace and rank to the proc struct they gave us */
-        (void)strncpy(proc->nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
+        pmix_strncpy(proc->nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
     }
     if (!rank_given) {
         proc->rank = pmix_globals.myid.rank;
@@ -621,36 +612,21 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
     pmix_globals.mypeer->info->pname.nspace = strdup(pmix_globals.myid.nspace);
     pmix_globals.mypeer->info->pname.rank = pmix_globals.myid.rank;
 
-    /* if we are acting as a client, then send a request for our
-     * job info - we do this as a non-blocking
-     * transaction because some systems cannot handle very large
-     * blocking operations and error out if we try them. */
-    if (PMIX_PROC_IS_CLIENT(pmix_globals.mypeer)) {
-         req = PMIX_NEW(pmix_buffer_t);
-         PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver,
-                          req, &cmd, 1, PMIX_COMMAND);
-         if (PMIX_SUCCESS != rc) {
-            PMIX_ERROR_LOG(rc);
-            PMIX_RELEASE(req);
-            PMIX_RELEASE_THREAD(&pmix_global_lock);
-            return rc;
-        }
-        /* send to the server */
-        PMIX_CONSTRUCT(&cb, pmix_cb_t);
-        PMIX_PTL_SEND_RECV(rc, pmix_client_globals.myserver,
-                           req, job_data, (void*)&cb);
-        if (PMIX_SUCCESS != rc) {
-            PMIX_RELEASE_THREAD(&pmix_global_lock);
-            return rc;
-        }
-        /* wait for the data to return */
-        PMIX_WAIT_THREAD(&cb.lock);
-        rc = cb.status;
-        PMIX_DESTRUCT(&cb);
-        if (PMIX_SUCCESS != rc) {
+    /* if we are acting as a server, then start listening */
+    if (PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
+        /* setup the wildcard recv for inbound messages from clients */
+        rcv = PMIX_NEW(pmix_ptl_posted_recv_t);
+        rcv->tag = UINT32_MAX;
+        rcv->cbfunc = pmix_server_message_handler;
+        /* add it to the end of the list of recvs */
+        pmix_list_append(&pmix_ptl_globals.posted_recvs, &rcv->super);
+        /* open the pnet framework so we can harvest envars */
+        rc = pmix_mca_base_framework_open(&pmix_pnet_base_framework, 0);
+        if (PMIX_SUCCESS != rc){
             PMIX_RELEASE_THREAD(&pmix_global_lock);
             return rc;
         }
+        /* note that we do not select active plugins as we don't need them */
     }
 
     /* setup IOF */
@@ -699,7 +675,7 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
                                &stdinev.ev, fd,
                                PMIX_EV_READ,
                                pmix_iof_read_local_handler, &stdinev);
-            }                                                               \
+            }
             /* check to see if we want the stdin read event to be
              * active - we will always at least define the event,
              * but may delay its activation
@@ -732,12 +708,42 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
     /* increment our init reference counter */
     pmix_globals.init_cntr++;
 
-    if (!PMIX_PROC_IS_CLIENT(pmix_globals.mypeer)) {
+    /* if we are acting as a client, then send a request for our
+     * job info - we do this as a non-blocking
+     * transaction because some systems cannot handle very large
+     * blocking operations and error out if we try them. */
+    if (PMIX_PROC_IS_CLIENT(pmix_globals.mypeer)) {
+         req = PMIX_NEW(pmix_buffer_t);
+         PMIX_BFROPS_PACK(rc, pmix_client_globals.myserver,
+                          req, &cmd, 1, PMIX_COMMAND);
+         if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+            PMIX_RELEASE(req);
+            PMIX_RELEASE_THREAD(&pmix_global_lock);
+            return rc;
+        }
+        /* send to the server */
+        PMIX_CONSTRUCT(&cb, pmix_cb_t);
+        PMIX_PTL_SEND_RECV(rc, pmix_client_globals.myserver,
+                           req, job_data, (void*)&cb);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_RELEASE_THREAD(&pmix_global_lock);
+            return rc;
+        }
+        /* wait for the data to return */
+        PMIX_WAIT_THREAD(&cb.lock);
+        rc = cb.status;
+        PMIX_DESTRUCT(&cb);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_RELEASE_THREAD(&pmix_global_lock);
+            return rc;
+        }
+    } else {
         /* now finish the initialization by filling our local
          * datastore with typical job-related info. No point
          * in having the server generate these as we are
          * obviously a singleton, and so the values are well-known */
-        (void)strncpy(wildcard.nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
+        pmix_strncpy(wildcard.nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
         wildcard.rank = pmix_globals.myid.rank;
 
         /* the jobid is just our nspace */
@@ -1045,13 +1051,6 @@ PMIX_EXPORT int PMIx_tool_init(pmix_proc_t *proc,
 
     /* if we are acting as a server, then start listening */
     if (PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
-        /* setup the wildcard recv for inbound messages from clients */
-        rcv = PMIX_NEW(pmix_ptl_posted_recv_t);
-        rcv->tag = UINT32_MAX;
-        rcv->cbfunc = pmix_server_message_handler;
-        /* add it to the end of the list of recvs */
-        pmix_list_append(&pmix_ptl_globals.posted_recvs, &rcv->super);
-
         /* start listening for connections */
         if (PMIX_SUCCESS != pmix_ptl_base_start_listening(info, ninfo)) {
             pmix_show_help("help-pmix-server.txt", "listener-thread-start", true);
@@ -1094,8 +1093,8 @@ static void finwait_cbfunc(struct pmix_peer_t *pr,
     if (tev->active) {
         tev->active = false;
         pmix_event_del(&tev->ev);  // stop the timer
-        PMIX_WAKEUP_THREAD(&tev->lock);
     }
+    PMIX_WAKEUP_THREAD(&tev->lock);
 }
 
 PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
@@ -1104,10 +1103,9 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
     pmix_cmd_t cmd = PMIX_FINALIZE_CMD;
     pmix_status_t rc;
     pmix_tool_timeout_t tev;
-    struct timeval tv = {2, 0};
+    struct timeval tv = {5, 0};
     int n;
     pmix_peer_t *peer;
-    pmix_setup_caddy_t *cd;
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
     if (1 != pmix_globals.init_cntr) {
@@ -1116,6 +1114,7 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
         return PMIX_SUCCESS;
     }
     pmix_globals.init_cntr = 0;
+    pmix_globals.mypeer->finalized = true;
     PMIX_RELEASE_THREAD(&pmix_global_lock);
 
     pmix_output_verbose(2, pmix_globals.debug_output,
@@ -1163,6 +1162,7 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
         /* wait for the ack to return */
         PMIX_WAIT_THREAD(&tev.lock);
         PMIX_DESTRUCT_LOCK(&tev.lock);
+
         if (tev.active) {
             pmix_event_del(&tev.ev);
         }
@@ -1179,7 +1179,7 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
         (void)pmix_progress_thread_pause(NULL);
     }
 
-    PMIX_RELEASE(pmix_client_globals.myserver);
+//    PMIX_RELEASE(pmix_client_globals.myserver);
     PMIX_LIST_DESTRUCT(&pmix_client_globals.pending_requests);
     for (n=0; n < pmix_client_globals.peers.size; n++) {
         if (NULL != (peer = (pmix_peer_t*)pmix_pointer_array_get_item(&pmix_client_globals.peers, n))) {
@@ -1190,19 +1190,13 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
     if (PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
         pmix_ptl_base_stop_listening();
 
-        /* cleanout any IOF */
-        for (n=0; n < PMIX_IOF_HOTEL_SIZE; n++) {
-            pmix_hotel_checkout_and_return_occupant(&pmix_server_globals.iof, n, (void**)&cd);
-            if (NULL != cd) {
-                PMIX_RELEASE(cd);
-            }
-        }
-        PMIX_DESTRUCT(&pmix_server_globals.iof);
         for (n=0; n < pmix_server_globals.clients.size; n++) {
             if (NULL != (peer = (pmix_peer_t*)pmix_pointer_array_get_item(&pmix_server_globals.clients, n))) {
                 PMIX_RELEASE(peer);
             }
         }
+
+        (void)pmix_mca_base_framework_close(&pmix_pnet_base_framework);
         PMIX_DESTRUCT(&pmix_server_globals.clients);
         PMIX_LIST_DESTRUCT(&pmix_server_globals.collectives);
         PMIX_LIST_DESTRUCT(&pmix_server_globals.remote_pnd);
@@ -1210,6 +1204,7 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
         PMIX_LIST_DESTRUCT(&pmix_server_globals.gdata);
         PMIX_LIST_DESTRUCT(&pmix_server_globals.events);
         PMIX_LIST_DESTRUCT(&pmix_server_globals.nspaces);
+        PMIX_LIST_DESTRUCT(&pmix_server_globals.iof);
     }
 
     /* shutdown services */
@@ -1220,6 +1215,7 @@ PMIX_EXPORT pmix_status_t PMIx_tool_finalize(void)
 
     /* finalize the class/object system */
     pmix_class_finalize();
+
     return PMIX_SUCCESS;
 }
 
