@@ -14,8 +14,8 @@
  * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -317,6 +317,17 @@ static void model_registration_callback(pmix_status_t status,
     DEBUG_WAKEUP_THREAD(lock);
 }
 
+static void set_handler_default(int sig)
+{
+    struct sigaction act;
+
+    act.sa_handler = SIG_DFL;
+    act.sa_flags = 0;
+    sigemptyset(&act.sa_mask);
+
+    sigaction(sig, &act, (struct sigaction *)0);
+}
+
 int main(int argc, char **argv)
 {
     char **client_env=NULL;
@@ -332,7 +343,6 @@ int main(int argc, char **argv)
     pmix_info_t *info;
     size_t ninfo;
     bool cross_version = false;
-    bool usock = true;
     bool hwloc = false;
 #if PMIX_HAVE_HWLOC
     char *hwloc_file = NULL;
@@ -346,8 +356,6 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-    fprintf(stderr, "Testing version %s\n", PMIx_Get_version());
-
     /* see if we were passed the number of procs to run or
      * the executable to use */
     for (n=1; n < argc; n++) {
@@ -370,25 +378,6 @@ int main(int argc, char **argv)
             /* cross-version test - we will set one child to
              * run at a different version. Requires -n >= 2 */
             cross_version = true;
-            usock = false;
-        } else if (0 == strcmp("-u", argv[n])) {
-            /* enable usock */
-            usock = false;
-#if PMIX_HAVE_HWLOC
-        } else if (0 == strcmp("-hwloc", argv[n]) ||
-                   0 == strcmp("--hwloc", argv[n])) {
-            /* test hwloc support */
-            hwloc = true;
-        } else if (0 == strcmp("-hwloc-file", argv[n]) ||
-                   0 == strcmp("--hwloc-file", argv[n])) {
-            if (NULL == argv[n+1]) {
-                fprintf(stderr, "The --hwloc-file option requires an argument\n");
-                exit(1);
-            }
-            hwloc_file = strdup(argv[n+1]);
-            hwloc = true;
-            ++n;
-#endif
         } else if (0 == strcmp("-h", argv[n])) {
             /* print the options and exit */
             fprintf(stderr, "usage: simptest <options>\n");
@@ -416,34 +405,35 @@ int main(int argc, char **argv)
     }
 #endif
 
+    fprintf(stderr, "Testing version %s\n", PMIx_Get_version());
+
     /* setup the server library and tell it to support tool connections */
 #if PMIX_HAVE_HWLOC
     if (hwloc) {
 #if HWLOC_API_VERSION < 0x20000
-        ninfo = 4;
+        ninfo = 3;
 #else
-        ninfo = 5;
+        ninfo = 4;
 #endif
     } else {
-        ninfo = 3;
+        ninfo = 2;
     }
 #else
-    ninfo = 3;
+    ninfo = 2;
 #endif
 
     PMIX_INFO_CREATE(info, ninfo);
     PMIX_INFO_LOAD(&info[0], PMIX_SERVER_TOOL_SUPPORT, NULL, PMIX_BOOL);
-    PMIX_INFO_LOAD(&info[1], PMIX_USOCK_DISABLE, &usock, PMIX_BOOL);
-    PMIX_INFO_LOAD(&info[2], PMIX_SERVER_GATEWAY, NULL, PMIX_BOOL);
+    PMIX_INFO_LOAD(&info[1], PMIX_SERVER_GATEWAY, NULL, PMIX_BOOL);
 #if PMIX_HAVE_HWLOC
     if (hwloc) {
         if (NULL != hwloc_file) {
-            PMIX_INFO_LOAD(&info[3], PMIX_TOPOLOGY_FILE, hwloc_file, PMIX_STRING);
+            PMIX_INFO_LOAD(&info[2], PMIX_TOPOLOGY_FILE, hwloc_file, PMIX_STRING);
         } else {
-            PMIX_INFO_LOAD(&info[3], PMIX_TOPOLOGY, NULL, PMIX_STRING);
+            PMIX_INFO_LOAD(&info[2], PMIX_TOPOLOGY, NULL, PMIX_STRING);
         }
 #if HWLOC_API_VERSION >= 0x20000
-        PMIX_INFO_LOAD(&info[4], PMIX_HWLOC_SHARE_TOPO, NULL, PMIX_BOOL);
+        PMIX_INFO_LOAD(&info[3], PMIX_HWLOC_SHARE_TOPO, NULL, PMIX_BOOL);
 #endif
     }
 #endif
@@ -549,9 +539,6 @@ int main(int argc, char **argv)
             } else {
                 pmix_setenv("PMIX_MCA_ptl", "usock", true, &client_env);
             }
-        } else if (!usock) {
-            /* don't disable usock => enable it on client */
-            pmix_setenv("PMIX_MCA_ptl", "usock", true, &client_env);
         }
         x = PMIX_NEW(myxfer_t);
         if (PMIX_SUCCESS != (rc = PMIx_server_register_client(&proc, myuid, mygid,
@@ -570,17 +557,24 @@ int main(int argc, char **argv)
             PMIx_server_finalize();
             return -1;
         }
-        child = PMIX_NEW(wait_tracker_t);
-        child->pid = pid;
-        pmix_list_append(&children, &child->super);
-
         if (pid == 0) {
+            sigset_t sigs;
+            set_handler_default(SIGTERM);
+            set_handler_default(SIGINT);
+            set_handler_default(SIGHUP);
+            set_handler_default(SIGPIPE);
+            set_handler_default(SIGCHLD);
+            sigprocmask(0, 0, &sigs);
+            sigprocmask(SIG_UNBLOCK, &sigs, 0);
             execve(executable, client_argv, client_env);
             /* Does not return */
             exit(0);
+        } else {
+            child = PMIX_NEW(wait_tracker_t);
+            child->pid = pid;
+            pmix_list_append(&children, &child->super);
         }
     }
-    free(executable);
     pmix_argv_free(client_argv);
     pmix_argv_free(client_env);
 
@@ -592,15 +586,21 @@ int main(int argc, char **argv)
         nanosleep(&ts, NULL);
     }
 
-    /* see if anyone exited with non-zero status */
-    n=0;
-    PMIX_LIST_FOREACH(child, &children, wait_tracker_t) {
-        if (0 != child->exit_code) {
-            fprintf(stderr, "Child %d exited with status %d - test FAILED\n", n, child->exit_code);
-            goto done;
-        }
-        ++n;
+    /* see if anyone exited with non-zero status unless the test
+     * was expected to do so */
+    if (NULL == strstr(executable, "simpdie")) {
+      n=0;
+      PMIX_LIST_FOREACH(child, &children, wait_tracker_t) {
+          if (0 != child->exit_code) {
+              fprintf(stderr, "Child %d [%d] exited with status %d - test FAILED\n", n, child->pid, child->exit_code);
+          }
+          ++n;
+      }
+    } else if (1 == exit_code) {
+      exit_code = 0;
     }
+    free(executable);
+
     /* try notifying ourselves */
     ninfo = 3;
     PMIX_INFO_CREATE(info, ninfo);
@@ -711,21 +711,14 @@ static void errhandler_reg_callbk (pmix_status_t status,
 static pmix_status_t connected(const pmix_proc_t *proc, void *server_object,
                                pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 static pmix_status_t finalized(const pmix_proc_t *proc, void *server_object,
                      pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
     pmix_output(0, "SERVER: FINALIZED %s:%d WAKEUP %d",
                 proc->nspace, proc->rank, wakeup);
-    /* ensure we call the cbfunc so the proc can exit! */
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static void abcbfunc(pmix_status_t status, void *cbdata)
@@ -783,17 +776,31 @@ static pmix_status_t abort_fn(const pmix_proc_t *proc,
     return PMIX_SUCCESS;
 }
 
+static void fencbfn(int sd, short args, void *cbdata)
+{
+    pmix_shift_caddy_t *scd = (pmix_shift_caddy_t*)cbdata;
 
+    /* pass the provided data back to each participating proc */
+    if (NULL != scd->cbfunc.modexcbfunc) {
+        scd->cbfunc.modexcbfunc(scd->status, scd->data, scd->ndata, scd->cbdata, NULL, NULL);
+    }
+    PMIX_RELEASE(scd);
+}
 static pmix_status_t fencenb_fn(const pmix_proc_t procs[], size_t nprocs,
                       const pmix_info_t info[], size_t ninfo,
                       char *data, size_t ndata,
                       pmix_modex_cbfunc_t cbfunc, void *cbdata)
 {
+    pmix_shift_caddy_t *scd;
+
     pmix_output(0, "SERVER: FENCENB");
-    /* pass the provided data back to each participating proc */
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, data, ndata, cbdata, NULL, NULL);
-    }
+    scd = PMIX_NEW(pmix_shift_caddy_t);
+    scd->status = PMIX_SUCCESS;
+    scd->data = data;
+    scd->ndata = ndata;
+    scd->cbfunc.modexcbfunc = cbfunc;
+    scd->cbdata = cbdata;
+    PMIX_THREADSHIFT(scd, fencbfn);
     return PMIX_SUCCESS;
 }
 
@@ -802,6 +809,8 @@ static pmix_status_t dmodex_fn(const pmix_proc_t *proc,
                      const pmix_info_t info[], size_t ninfo,
                      pmix_modex_cbfunc_t cbfunc, void *cbdata)
 {
+    pmix_shift_caddy_t *scd;
+
     pmix_output(0, "SERVER: DMODEX");
 
     /* if this is a timeout test, then do nothing */
@@ -809,11 +818,12 @@ static pmix_status_t dmodex_fn(const pmix_proc_t *proc,
         return PMIX_SUCCESS;
     }
 
-    /* we don't have any data for remote procs as this
-     * test only runs one server - so report accordingly */
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_ERR_NOT_FOUND, NULL, 0, cbdata, NULL, NULL);
-    }
+    scd = PMIX_NEW(pmix_shift_caddy_t);
+    scd->status = PMIX_ERR_NOT_FOUND;
+    scd->cbfunc.modexcbfunc = cbfunc;
+    scd->cbdata = cbdata;
+    PMIX_THREADSHIFT(scd, fencbfn);
+
     return PMIX_SUCCESS;
 }
 
@@ -835,12 +845,26 @@ static pmix_status_t publish_fn(const pmix_proc_t *proc,
         pmix_value_xfer(&p->pdata.value, (pmix_value_t*)&info[n].value);
         pmix_list_append(&pubdata, &p->super);
     }
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-    return PMIX_SUCCESS;
+
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
+typedef struct {
+    pmix_event_t ev;
+    pmix_pdata_t *pd;
+    size_t n;
+    pmix_lookup_cbfunc_t cbfunc;
+    void *cbdata;
+} lkobj_t;
+
+static void lkcbfn(int sd, short args, void *cbdata)
+{
+    lkobj_t *lk = (lkobj_t*)cbdata;
+
+    lk->cbfunc(PMIX_SUCCESS, lk->pd, lk->n, lk->cbdata);
+    PMIX_PDATA_FREE(lk->pd, lk->n);
+    free(lk);
+}
 
 static pmix_status_t lookup_fn(const pmix_proc_t *proc, char **keys,
                      const pmix_info_t info[], size_t ninfo,
@@ -851,6 +875,7 @@ static pmix_status_t lookup_fn(const pmix_proc_t *proc, char **keys,
     size_t i, n;
     pmix_pdata_t *pd = NULL;
     pmix_status_t ret = PMIX_ERR_NOT_FOUND;
+    lkobj_t *lk;
 
     pmix_output(0, "SERVER: LOOKUP");
 
@@ -883,13 +908,16 @@ static pmix_status_t lookup_fn(const pmix_proc_t *proc, char **keys,
         }
     }
     PMIX_LIST_DESTRUCT(&results);
-    if (NULL != cbfunc) {
-        cbfunc(ret, pd, n, cbdata);
+    if (PMIX_SUCCESS == ret) {
+        lk = (lkobj_t*)malloc(sizeof(lkobj_t));
+        lk->pd = pd;
+        lk->n = n;
+        lk->cbfunc = cbfunc;
+        lk->cbdata = cbdata;
+        PMIX_THREADSHIFT(lk, lkcbfn);
     }
-    if (0 < n) {
-        PMIX_PDATA_FREE(pd, n);
-    }
-    return PMIX_SUCCESS;
+
+    return ret;
 }
 
 
@@ -911,10 +939,7 @@ static pmix_status_t unpublish_fn(const pmix_proc_t *proc, char **keys,
             }
         }
     }
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static void spcbfunc(pmix_status_t status, void *cbdata)
@@ -978,11 +1003,7 @@ static pmix_status_t connect_fn(const pmix_proc_t procs[], size_t nprocs,
 
     numconnects++;
 
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 
@@ -992,30 +1013,20 @@ static pmix_status_t disconnect_fn(const pmix_proc_t procs[], size_t nprocs,
 {
     pmix_output(0, "SERVER: DISCONNECT");
 
-    /* in practice, we would pass this request to the local
-     * resource manager for handling */
-
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static pmix_status_t register_event_fn(pmix_status_t *codes, size_t ncodes,
                                        const pmix_info_t info[], size_t ninfo,
                                        pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static pmix_status_t deregister_events(pmix_status_t *codes, size_t ncodes,
                                        pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static pmix_status_t notify_event(pmix_status_t code,
@@ -1024,14 +1035,26 @@ static pmix_status_t notify_event(pmix_status_t code,
                                   pmix_info_t info[], size_t ninfo,
                                   pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
-    return PMIX_SUCCESS;
+    pmix_output(0, "SERVER: NOTIFY EVENT");
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 typedef struct query_data_t {
+    pmix_event_t ev;
     pmix_info_t *data;
     size_t ndata;
+    pmix_info_cbfunc_t cbfunc;
+    void *cbdata;
 } query_data_t;
 
+static void qfn(int sd, short args, void *cbdata)
+{
+    query_data_t *qd = (query_data_t*)cbdata;
+
+    qd->cbfunc(PMIX_SUCCESS, qd->data, qd->ndata, qd->cbdata, NULL, NULL);
+    PMIX_INFO_FREE(qd->data, qd->ndata);
+}
+
 static pmix_status_t query_fn(pmix_proc_t *proct,
                               pmix_query_t *queries, size_t nqueries,
                               pmix_info_cbfunc_t cbfunc,
@@ -1039,6 +1062,7 @@ static pmix_status_t query_fn(pmix_proc_t *proct,
 {
     size_t n;
     pmix_info_t *info;
+    query_data_t qd;
 
     pmix_output(0, "SERVER: QUERY");
 
@@ -1055,7 +1079,11 @@ static pmix_status_t query_fn(pmix_proc_t *proct,
             return PMIX_ERROR;
         }
     }
-    cbfunc(PMIX_SUCCESS, info, nqueries, cbdata, NULL, NULL);
+    qd.data = info;
+    qd.ndata = nqueries;
+    qd.cbfunc = cbfunc;
+    qd.cbdata = cbdata;
+    PMIX_THREADSHIFT(&qd, qfn);
     return PMIX_SUCCESS;
 }
 
@@ -1076,16 +1104,29 @@ static void tool_connect_fn(pmix_info_t *info, size_t ninfo,
     }
 }
 
+typedef struct {
+    pmix_event_t ev;
+    pmix_op_cbfunc_t cbfunc;
+    void *cbdata;
+} mylog_t;
+
+static void foobar(int sd, short args, void *cbdata)
+{
+    mylog_t *lg = (mylog_t*)cbdata;
+    lg->cbfunc(PMIX_SUCCESS, lg->cbdata);
+}
 static void log_fn(const pmix_proc_t *client,
                    const pmix_info_t data[], size_t ndata,
                    const pmix_info_t directives[], size_t ndirs,
                    pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
+    mylog_t *lg = (mylog_t *)malloc(sizeof(mylog_t));
+
     pmix_output(0, "SERVER: LOG");
 
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, cbdata);
-    }
+    lg->cbfunc = cbfunc;
+    lg->cbdata = cbdata;
+    PMIX_THREADSHIFT(lg, foobar);
 }
 
 static pmix_status_t alloc_fn(const pmix_proc_t *client,
@@ -1093,7 +1134,7 @@ static pmix_status_t alloc_fn(const pmix_proc_t *client,
                               const pmix_info_t data[], size_t ndata,
                               pmix_info_cbfunc_t cbfunc, void *cbdata)
 {
-    return PMIX_SUCCESS;
+    return PMIX_OPERATION_SUCCEEDED;
 }
 
 static pmix_status_t jctrl_fn(const pmix_proc_t *requestor,
@@ -1141,14 +1182,21 @@ static void wait_signal_callback(int fd, short event, void *arg)
         /* we are already in an event, so it is safe to access the list */
         PMIX_LIST_FOREACH(t2, &children, wait_tracker_t) {
             if (pid == t2->pid) {
-                t2->exit_code = status;
                 /* found it! */
-                if (0 != status && 0 == exit_code) {
-                    exit_code = status;
+                if (WIFEXITED(status)) {
+                    t2->exit_code = WEXITSTATUS(status);
+                } else {
+                    if (WIFSIGNALED(status)) {
+                        t2->exit_code = WTERMSIG(status) + 128;
+                    }
+                }
+                if (0 != t2->exit_code && 0 == exit_code) {
+                    exit_code = t2->exit_code;
                 }
                 --wakeup;
                 break;
             }
         }
     }
+    fprintf(stderr, "ENDLOOP\n");
 }
